@@ -147,19 +147,6 @@ dm_is_safe() {
 	return 0
 }
 
-loop_is_safe() {
-	local opts
-	type losetup-aes > /dev/null 2>&1 || return 1
-
-	opts=$(losetup-aes $1 2>&1)
-	if [ $? -eq 0 ] && echo "$opts" | grep -q encryption=; then
-		# loop entry has an encryption= option, assume it's safe
-		return 0
-	fi
-
-	return 1
-}
-
 swap_is_safe () {
 	local swap
 	local IFS="
@@ -167,12 +154,9 @@ swap_is_safe () {
 
 	for swap in $(cat /proc/swaps); do
 		case $swap in
-		    Filename*)
+		    Filename*|/dev/zram*)
 			continue
 			;;
-		    /dev/loop*)
-			loop_is_safe ${swap%% *} || return 1
-			;;
 		    /dev/mapper/*)
 			dm_is_safe ${swap%% *} || return 1
 			;;
@@ -186,26 +170,6 @@ swap_is_safe () {
 	return 0
 }
 
-get_free_loop () {
-	O=$IFS
-	IFS="
-"
-	for n in $(losetup-aes -a); do
-		n=${n%%:*}
-		n=${n#/dev/loop}
-		n=${n#/}
-		eval loop$n=1
-	done
-	IFS=$O
-
-	for n in 0 1 2 3 4 5 6 7; do
-		if eval [ -z "\$loop$n" ]; then
-			echo /dev/loop$n
-			break
-		fi
-	done
-}
-
 get_free_mapping() {
 	for n in 0 1 2 3 4 5 6 7; do
 		if [ ! -b "/dev/mapper/crypt$n" ]; then
@@ -215,37 +179,6 @@ get_free_mapping() {
 	done
 }
 
-setup_loopaes () {
-	local loop device cipher keytype keyfile opts pass
-	loop=$1
-	device=$2
-	cipher=$3
-	keytype=$4
-	keyfile=$5
-
-	[ -x /sbin/losetup-aes ] || return 1
-
-	case $keytype in
-	    keyfile)
-		opts="-K $keyfile"
-		pass="$keyfile.pass"
-		;;
-	    random)
-		opts="-H random"
-		pass="/dev/null"
-		;;
-	esac
-
-	log-output -t partman-crypto \
-	/sbin/losetup-aes -e $cipher $opts -p0 -G / $loop $device < $pass
-	if [ $? -ne 0 ]; then
-		log "losetup failed"
-		return 2
-	fi
-
-	return 0
-}
-
 setup_dmcrypt () {
 	local mapping device cipher iv hash size pass
 	mapping=$1
@@ -272,24 +205,44 @@ setup_dmcrypt () {
 }
 
 setup_luks () {
-	local mapping device cipher iv size pass
+	local mapping device cipher iv size pass luksopts
 	mapping=$1
 	device=$2
 	cipher=$3
 	iv=$4
-	size=$5
-	pass=$6
+	hash=$5
+	size=$6
+	pass=$7
 
 	[ -x /sbin/cryptsetup ] || return 1
 
 	# xts modes needs double the key size
 	[ "${iv%xts-*}" = "${iv}" ] || size="$(($size * 2))"
 
-	log-output -t partman-crypto \
-	/sbin/cryptsetup -c $cipher-$iv -s $size luksFormat $device $pass
-	if [ $? -ne 0 ]; then
-		log "luksFormat failed"
-		return 2
+	zkey_available=0
+	if [ -x /usr/bin/zkey ] ; then
+		log-output -t partman-crypto zkey generate --name $mapping --xts --volumes $device:$mapping --volume-type luks2 --sector-size 4096
+		if [ $? -eq 0 ]; then
+			log-output -t partman-crypto zkey cryptsetup --run --volumes $device --batch-mode --key-file $pass
+			if [ $? -eq 0 ]; then
+				zkey_available=1
+			fi
+		fi
+		if [ $zkey_available -eq 0 ]; then
+			log "zkey generate and cryptsetup failed, assuming not available."
+		fi
+	fi
+
+	if [ $zkey_available -eq 0 ]; then
+		if db_get partman-crypto/luksformat_options; then
+			luksopts="$RET"
+			log "Additional options for luksFormat: '$luksopts'"
+		fi
+		log-output -t partman-crypto /sbin/cryptsetup -c $cipher-$iv -h $hash -s $size $luksopts luksFormat $device $pass
+		if [ $? -ne 0 ]; then
+			log "luksFormat failed"
+			return 2
+		fi
 	fi
 
 	log-output -t partman-crypto \
@@ -329,7 +282,7 @@ setup_cryptdev () {
 			fi
 		fi
 		if [ $keytype = passphrase ]; then
-			setup_luks $cryptdev $realdev $cipher $ivalgorithm $keysize $keyfile || return 1
+			setup_luks $cryptdev $realdev $cipher $ivalgorithm $keyhash $keysize $keyfile || return 1
 		elif [ $keytype = random ]; then
 			setup_dmcrypt $cryptdev $realdev $cipher $ivalgorithm plain $keysize /dev/urandom || return 1
 		else
@@ -338,14 +291,6 @@ setup_cryptdev () {
 		cryptdev="/dev/mapper/$cryptdev"
 		;;
 
-	    loop-AES)
-		cryptdev=$(get_free_loop);
-		if [ -z "$cryptdev" ]; then
-			return 1
-		fi
-		setup_loopaes $cryptdev $realdev $cipher $keytype $keyfile || return 1
-		;;
-
 	esac
 
 	echo $cryptdev > $id/crypt_active
@@ -362,12 +307,13 @@ crypto_do_wipe () {
 	fifo=/var/run/wipe_progress
 
 	mknod $fifo p
-	/bin/blockdev-wipe -s 65536 $dev > $fifo &
+	/bin/blockdev-wipe -s $((512*1024)) $dev > $fifo &
 	pid=$!
 
 	cancelled=0
 	db_capb backup align progresscancel
-	db_progress START 0 65536 $template
+	db_progress START 0 1000 ${template}_title
+	db_progress INFO ${template}_text
 	while read x <&9; do
 		db_progress STEP 1
 		if [ $? -eq 30 ]; then
@@ -388,19 +334,24 @@ crypto_do_wipe () {
 }
 
 crypto_wipe_device () {
-	local device size method interactive targetdevice
+	local device part interactive type cipher ivalgorithm keysize targetdevice
 	device=$1
-	size=$2
-	method=$3
-	interactive=$4
+	part=$2
+	interactive=$3
 	if [ "$interactive" != no ]; then
 		interactive=yes
 	fi
 	ret=1
 
+	if [ -r $part/crypto_type ] && [ "$(cat $part/crypto_type)" = dm-crypt ]; then
+		type=crypto
+	else
+		type=plain
+	fi
+
 	if [ $interactive = yes ]; then
 		# Confirm before erasing
-		template="partman-crypto/warn_erase"
+		template="partman-crypto/${type}_warn_erase"
 		db_set $template false
 		db_subst $template DEVICE $(humandev $device)
 		db_input critical $template || true
@@ -412,23 +363,26 @@ crypto_wipe_device () {
 	fi
 
 	# Setup crypto
-	if [ $method = loop-AES ]; then
-		targetdevice=$(get_free_loop)
-		setup_loopaes $targetdevice $device AES128 random || return 1
-	elif [ $method = dm-crypt ]; then
+	if [ "$type" = crypto ]; then
+		cipher=$(cat $part/cipher)
+		ivalgorithm=$(cat $part/ivalgorithm)
+		keysize=$(cat $part/keysize)
 		targetdevice=$(get_free_mapping)
-		setup_dmcrypt $targetdevice $device aes xts-plain64 plain 128 /dev/urandom || return 1
+		setup_dmcrypt $targetdevice $device $cipher $ivalgorithm plain $keysize /dev/urandom || return 1
 		targetdevice="/dev/mapper/$targetdevice"
+		log "wiping $targetdevice with $cipher $ivalgorithm $keysize"
 	else
 		# Just wipe the device with zeroes
 		targetdevice=$device
+		log "wiping $targetdevice with plain zeroes"
 	fi
 
 	# Erase
-	template="partman-crypto/progress/erase"
-	db_subst $template DEVICE $(humandev $device)
+	template="partman-crypto/progress/${type}_erase"
+	db_subst ${template}_title DEVICE $(humandev $device)
+	db_subst ${template}_text DEVICE $(humandev $device)
 	if ! crypto_do_wipe $template $targetdevice; then
-		template="partman-crypto/erase_failed"
+		template="partman-crypto/${type}_erase_failed"
 		db_subst $template DEVICE $(humandev $device)
 		db_input critical $template || true
 		db_go
@@ -437,9 +391,7 @@ crypto_wipe_device () {
 	fi
 
 	# Teardown crypto
-	if [ $method = loop-AES ]; then
-		log-output -t partman-crypto /sbin/losetup-aes -d $targetdevice
-	elif [ $method = dm-crypt ]; then
+	if [ "$type" = crypto ]; then
 		log-output -t partman-crypto /sbin/cryptsetup remove ${targetdevice##/dev/mapper/}
 	fi
 
@@ -506,38 +458,6 @@ crypto_dooption () {
 	echo $RET > $part/$option
 }
 
-# Unmount CD image (if any); switch to loop-aes module; remount
-# As we don't have losetup available, we cannot see what's mounted,
-# so assume loop is only used for the CD image in hd-media installs
-crypto_reload_loop() {
-	local iso_image iso_mp
-
-	if db_get iso-scan/filename && [ "$RET" ]; then
-		iso_image=$RET
-		iso_mp=$(grep "^/dev/loop.*iso9660" /proc/mounts | \
-			 head -n1 | cut -d" " -f2)
-		if [ "$iso_mp" ]; then
-			log-output -t partman-crypto umount $iso_mp || return $?
-		fi
-	fi
-
-	RET=0
-	if log-output -t partman-crypto modprobe -r loop; then
-		modprobe -q loop-aes || return $?
-		sleep 1
-	else
-		RET=$?
-	fi
-
-	if [ "$iso_mp" ]; then
-		log "remounting CD image"
-		log-output -t partman-crypto \
-			mount -t iso9660 -o loop,ro,exec $iso_image $iso_mp || return $?
-	fi
-
-	return $RET
-}
-
 crypto_load_module() {
 	local module=$1
 
@@ -553,23 +473,13 @@ crypto_load_module() {
 		fi
 		modprobe -q $module
 		return $?
-	elif [ "$module" != loop-aes ]; then
+	else
 		if egrep -q "^(name|version) *: $module\$" /proc/crypto; then
 			return 0
 		fi
 		modprobe -q $module
 		return $?
 	fi
-
-	# loop-aes cannot be loaded if loop is already loaded (hd-media installs)
-	if ! log-output -t partman-crypto modprobe --first-time $module; then
-		log "failed to load module loop-aes; loop already loaded?"
-
-		if ! crypto_reload_loop; then
-			log "error: failed to replace already loaded module loop with loop-aes"
-			return 1
-		fi
-	fi
 }
 
 # Loads all modules for a given crypto type and cipher
@@ -684,18 +594,16 @@ crypto_set_defaults () {
 
 	case $type in
 	    dm-crypt)
-		echo aes > $part/cipher
-		echo 256 > $part/keysize
-		echo xts-plain64 > $part/ivalgorithm
-		echo passphrase > $part/keytype
-		echo sha256 > $part/keyhash
-		;;
-	    loop-AES)
-		echo AES256 > $part/cipher
-		echo keyfile > $part/keytype
-		rm -f $part/keysize
-		rm -f $part/ivalgorithm
-		rm -f $part/keyhash
+		db_get partman-crypto/cipher
+		echo ${RET:-aes} > $part/cipher
+		db_get partman-crypto/keysize
+		echo ${RET:-256} > $part/keysize
+		db_get partman-crypto/ivalgorithm
+		echo ${RET:-xts-plain64} > $part/ivalgorithm
+		db_get partman-crypto/keytype
+		echo ${RET:-passphrase} > $part/keytype
+		db_get partman-crypto/keyhash
+		echo ${RET:-sha256} > $part/keyhash
 		;;
 	esac
 	touch $part/skip_erase
@@ -715,9 +623,6 @@ crypto_prepare_method () {
 	    dm-crypt)
 		packages="$packages partman-crypto-dm"
 		;;
-	    loop-AES)
-		packages="$packages partman-crypto-loop"
-		;;
 	    *)
 		return 1
 		;;
@@ -747,9 +652,6 @@ crypto_check_required_tools() {
 	    dm-crypt)
 		tools="$tools dmsetup cryptsetup"
 		;;
-	    loop-AES)
-		tools="$tools gpg base64"
-		;;
 	    *)
 		return 1
 	esac
@@ -774,9 +676,6 @@ crypto_check_required_options() {
 	    dm-crypt)
 		options="cipher keytype keyhash ivalgorithm keysize"
 		;;
-	    loop-AES)
-		options="cipher keytype"
-		;;
 	esac
 
 	list=""
@@ -892,7 +791,7 @@ crypto_setup() {
 				continue
 			fi
 
-			if ! crypto_wipe_device $path $size $(cat $id/crypto_type) $interactive; then
+			if ! crypto_wipe_device $path $dev/$id $interactive; then
 				db_fset partman-crypto/commit_failed seen false
 				db_input critical partman-crypto/commit_failed
 				db_go || true
@@ -901,7 +800,7 @@ crypto_setup() {
 		done
 	done
 
-	# Create keys and do losetup/dmsetup
+	# Create keys and do dmsetup
 	for dev in $DEVICES/*; do
 		[ -d "$dev" ] || continue
 		cd $dev
@@ -935,9 +834,6 @@ crypto_setup() {
 			if [ $keytype = keyfile ] || [ $keytype = passphrase ]; then
 				keyfile=$(mapdevfs $path | tr / _)
 				keyfile="$dev/$id/${keyfile#_dev_}"
-				if [ $type = loop-AES ]; then
-					keyfile="${keyfile}.gpg"
-				fi
 
 				if [ ! -f $keyfile ]; then
 					if ! /bin/blockdev-keygen "$(humandev $path)" "$keytype" "$keyfile"; then
