@@ -227,9 +227,14 @@ class Install(install_misc.InstallBase):
         # Configure zsys
         self.configure_zsys()
 
+        self.next_region()
+        self.db.progress('INFO', 'ubiquity/install/activedirectory')
+        self.configure_active_directory()
+
         self.next_region()
         self.db.progress('INFO', 'ubiquity/install/bootloader')
         self.copy_mok()
+        self.configure_recovery_key()
         self.configure_bootloader()
 
         self.next_region(size=4)
@@ -895,6 +900,65 @@ class Install(install_misc.InstallBase):
                         continue
                 os.symlink(linksrc, linkdst)
 
+    def configure_recovery_key(self):
+        crypto_key = self.db.get('ubiquity/crypto_key')
+        recovery_key = self.db.get('ubiquity/recovery_key')
+        if not crypto_key or not recovery_key:
+            self.clean_crypto_keys()
+            return
+
+        debconf_disk = self.db.get('partman-auto/select_disk')
+        disk = debconf_disk.split('/')[-1].replace('=', '/')
+        if not disk:  # disk is not set in manual partitioning mode
+            syslog.syslog(
+                syslog.LOG_ERR,
+                'Determining installation disk failed. '
+                'Setting a recovery key is supported only with partman-auto.')
+            self.clean_crypto_keys()
+            self.db.input('critical', 'ubiquity/install/broken_luks_add_key')
+            self.db.go()
+            return
+
+        args = ['lsblk', '-lp', '-oNAME,FSTYPE', disk]
+        lsblk_out = subprocess.check_output(args).decode(sys.stdout.encoding)
+        for line in lsblk_out.splitlines():
+            if 'crypto_LUKS' not in line:
+                continue
+            dev = line.split()[0]
+
+        if not dev:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(args))
+            syslog.syslog(syslog.LOG_ERR, 'determining crypto device failed. Output: %s' % lsblk_out)
+            self.clean_crypto_keys()
+            self.db.input('critical', 'ubiquity/install/broken_luks_add_key')
+            self.db.go()
+            return
+        syslog.syslog(' '.join(args))
+
+        key_args = "%s\n%s" % (crypto_key, recovery_key)
+        try:
+            log_args = ['log-output', '-t', 'ubiquity']
+            log_args.extend(['cryptsetup', 'luksAddKey', dev])
+            p = subprocess.run(log_args, input=key_args, encoding="utf-8")
+        except subprocess.CalledProcessError as e:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
+            syslog.syslog(syslog.LOG_ERR, "cryptsetup failed(%s): %s" % (e.returncode, e.output))
+            return
+        finally:
+            self.clean_crypto_keys()
+
+        if p.returncode != 0:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
+            self.db.input('critical', 'ubiquity/install/broken_luks_add_key')
+            self.db.go()
+            return
+
+        syslog.syslog(' '.join(log_args))
+
+    def clean_crypto_keys(self):
+        self.db.set('ubiquity/crypto_key', '')
+        self.db.set('ubiquity/recovery_key', '')
+
     def configure_bootloader(self):
         """Configure and install the boot loader."""
         if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
@@ -961,6 +1025,74 @@ class Install(install_misc.InstallBase):
         if use_zfs:
             misc.execute_root('/usr/share/ubiquity/zsys-setup', 'finalize')
 
+    def configure_active_directory(self):
+        """ Join Active Directory domain and enable pam_mkhomedir """
+        use_directory = self.db.get('ubiquity/login_use_directory')
+        if use_directory != 'true':
+            install_misc.record_removed(['adcli', 'realmd', 'sssd'], recursive=True)
+            return
+
+        from socket import gethostname
+        hostname_cur = gethostname()
+        hostname_new = ''
+        with open(self.target_file('etc/hostname'), 'r') as f:
+            hostname_new = f.read().strip()
+
+        # Set hostname for AD to determine FQDN (no fqdn option in realm join, only adcli)
+        misc.execute_root('hostname', hostname_new)
+
+        directory_domain = self.db.get('ubiquity/directory_domain')
+        directory_user = self.db.get('ubiquity/directory_user')
+        directory_passwd = self.db.get('ubiquity/directory_passwd')
+
+        binds = ("/proc", "/sys", "/dev", "/run")
+        try:
+            for bind in binds:
+                misc.execute('mount', '--bind', bind, self.target + bind)
+            # join AD on host (services are running on host)
+            if not self.join_domain(hostname_new, directory_domain, directory_user, directory_passwd):
+                self.db.input('critical', 'ubiquity/install/broken_active_directory')
+                self.db.go()
+            install_misc.record_removed(['adcli'], recursive=True)
+        finally:
+            for bind in binds:
+                misc.execute('umount', '-f', self.target + bind)
+            # Reset hostname
+            misc.execute_root('hostname', hostname_cur)
+
+        # Enable pam_mkhomedir
+        try:
+            subprocess.check_call(['chroot', self.target, 'pam-auth-update',
+                                   '--package', '--enable', 'mkhomedir'],
+                                  preexec_fn=install_misc.debconf_disconnect)
+        except subprocess.CalledProcessError:
+            self.db.input('critical', 'ubiquity/install/broken_active_directory')
+            self.db.go()
+
+    def join_domain(self, hostname, directory_domain, directory_user, directory_passwd):
+        """ Join an Active Directory domain """
+        log_args = ['log-output', '-t', 'ubiquity']
+        log_args.extend(['realm', 'join', '--install', self.target,
+                         '--user', directory_user, '--computer-name', hostname,
+                         '--unattended', directory_domain])
+        try:
+            p = subprocess.run(log_args, input=directory_passwd, timeout=60, encoding="utf-8")
+        except TimeoutError as e:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
+            syslog.syslog(syslog.LOG_ERR, "Command timed out(%s): %s" % (e.errno, e.strerror))
+            return False
+        except IOError as e:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
+            syslog.syslog(syslog.LOG_ERR, "OS error(%s): %s" % (e.errno, e.strerror))
+            return False
+
+        if p.returncode != 0:
+            syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
+            return False
+
+        syslog.syslog(' '.join(log_args))
+        return True
+
     def copy_mok(self):
         if 'UBIQUITY_OEM_USER_CONFIG' in os.environ:
             return
