@@ -177,7 +177,10 @@ class Wizard(BaseFrontend):
         # Let's not do that here, we kind of really do need more privileges...
         QtCore.QCoreApplication.setSetuidAllowed(True)
 
-        self.app = QtWidgets.QApplication([])
+        # NB: This *must* have at least one string in argv. Quoting the Qt docs
+        #   > In addition, argc must be greater than zero and argv must contain
+        #   > at least one valid character string.
+        self.app = QtWidgets.QApplication(sys.argv)
         # The "hicolor" icon theme gets picked when Ubiquity is running as a
         # DM. This causes some icons to be missing. Hardcode the theme name to
         # prevent that.
@@ -304,6 +307,7 @@ class Wizard(BaseFrontend):
         self.installing_no_return = False
         self.partitioned = False
         self.timezone_set = False
+        self.ubuntu_drivers = None
         self.returncode = 0
         self.backup = False
         self.history = []
@@ -947,7 +951,8 @@ class Wizard(BaseFrontend):
 
         if self.pagesindex == 0:
             self.allow_go_backward(False)
-        elif self.pages[self.pagesindex - 1].module.NAME == 'partman':
+        elif 'partman' in [page.module.NAME for page in
+                           self.pages[:self.pagesindex - 1]]:
             # We're past partitioning.  Unless the install fails, there is no
             # going back.
             self.allow_go_backward(False)
