@@ -35,7 +35,7 @@ WEIGHT = 10
 try:
     import lsb_release
     _ver = lsb_release.get_distro_information()['RELEASE']
-except:
+except Exception:
     _ver = '12.04'
 _wget_url = 'http://changelogs.ubuntu.com/ubiquity/%s-update-available' % _ver
 
@@ -61,9 +61,6 @@ class PageBase(plugin.PluginUI):
     def get_oem_id(self):
         return ''
 
-    def set_alpha_warning(self, show):
-        self.show_alpha_warning = show
-
 
 class PageGtk(PageBase):
     plugin_is_language = True
@@ -109,12 +106,12 @@ class PageGtk(PageBase):
                 self.release_notes_found = True
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except:
+            except Exception:
                 pass
         self.install_ubuntu = builder.get_object('install_ubuntu')
         self.try_ubuntu = builder.get_object('try_ubuntu')
         if not self.only:
-            if not 'UBIQUITY_GREETER' in os.environ:
+            if 'UBIQUITY_GREETER' not in os.environ:
                 choice_section_vbox = builder.get_object('choice_section_vbox')
                 choice_section_vbox and choice_section_vbox.hide()
             else:
@@ -376,10 +373,8 @@ class PageGtk(PageBase):
                     self.release_notes_label.hide()
             self.updating_installer = False
         elif uri == 'release-notes':
-            import subprocess
             uri = self.release_notes_url.replace('${LANG}', lang)
-            subprocess.Popen(['sensible-browser', uri], close_fds=True,
-                             preexec_fn=misc.drop_all_privileges)
+            misc.launch_uri(uri)
         return True
 
 
@@ -397,29 +392,35 @@ class PageKde(PageBase):
             self.only = False
 
         try:
-            from PyQt4 import uic
-            from PyQt4.QtGui import QWidget, QPixmap
+            from PyQt5 import uic
+            from PyQt5.QtGui import QPixmap, QIcon
+            from PyQt5.QtWidgets import QWidget
             self.page = uic.loadUi('/usr/share/ubiquity/qt/stepLanguage.ui')
             self.combobox = self.page.language_combobox
+            # Tell layout that all items are of uniform sizes
+            # Fixes LP:1187762
+            self.combobox.view().setUniformItemSizes(True)
             self.combobox.currentIndexChanged[str].connect(
                 self.on_language_selection_changed)
             if not self.controller.oem_config:
-                self.page.oem_id_label.hide()
-                self.page.oem_id_entry.hide()
+                self.page.oem_widget.hide()
+
+            def init_big_button(button, image_name):
+                pix = QPixmap('/usr/share/ubiquity/qt/images/' + image_name)
+                icon = QIcon(pix)
+                button.setIcon(icon)
+                button.setIconSize(pix.size())
+                # Set a fixed height to ensure the button text is not cropped
+                # when the window is resized
+                button.setFixedHeight(button.sizeHint().height())
 
             def inst(*args):
                 self.page.try_ubuntu.setEnabled(False)
                 self.controller.go_forward()
             self.page.install_ubuntu.clicked.connect(inst)
             self.page.try_ubuntu.clicked.connect(self.on_try_ubuntu_clicked)
-            picture1 = QPixmap(
-                "/usr/share/ubiquity/pixmaps/kubuntu-live-session.png")
-            self.page.image1.setPixmap(picture1)
-            self.page.image1.resize(picture1.size())
-            picture2 = QPixmap(
-                "/usr/share/ubiquity/pixmaps/kubuntu-install.png")
-            self.page.image2.setPixmap(picture2)
-            self.page.image2.resize(picture2.size())
+            init_big_button(self.page.install_ubuntu, 'install.png')
+            init_big_button(self.page.try_ubuntu, 'try.png')
 
             self.release_notes_url = ''
             self.update_installer = True
@@ -433,7 +434,7 @@ class PageKde(PageBase):
                 self.release_notes_found = True
             except (KeyboardInterrupt, SystemExit):
                 raise
-            except:
+            except Exception:
                 pass
 
             if self.release_notes_url:
@@ -442,15 +443,11 @@ class PageKde(PageBase):
             else:
                 self.page.release_notes_label.hide()
 
-            if not 'UBIQUITY_GREETER' in os.environ:
+            if 'UBIQUITY_GREETER' not in os.environ:
                 self.page.try_ubuntu.hide()
                 self.page.try_install_text_label.hide()
                 self.page.install_ubuntu.hide()
-                self.page.image1.hide()
-                self.page.image2.hide()
 
-            if self.only:
-                self.page.alpha_warning_label.hide()
             # We do not want to show the yet to be substituted strings
             # (${MEDIUM}, etc), so don't show the core of the page until
             # it's ready.
@@ -479,12 +476,6 @@ class PageKde(PageBase):
         self.controller._wizard.current_page = None
         self.controller.dbfilter.ok_handler()
 
-    def set_alpha_warning(self, show):
-        if not show and not self.only:
-            self.page.alpha_warning_label.hide()
-            if self.page.alpha_warning_label in self.widgetHidden:
-                self.widgetHidden.remove(self.page.alpha_warning_label)
-
     def on_release_notes_link(self, link):
         lang = self.selected_language()
         if link == "release-notes":
@@ -503,20 +494,27 @@ class PageKde(PageBase):
             self.updating_installer = False
 
     def openURL(self, url):
-        from PyQt4.QtGui import QDesktopServices
-        from PyQt4.QtCore import QUrl
-        from ubiquity.misc import drop_privileges_save, regain_privileges_save
+        from PyQt5.QtGui import QDesktopServices
+        from PyQt5.QtCore import QUrl
+        import shutil
+        import os
 
         # this nonsense is needed because kde doesn't want to be root
-        drop_privileges_save()
+        misc.drop_privileges()
+        misc.drop_privileges_save()
+        # copy over gtkrc-2.0 to get the themeing right
+        if os.path.exists("/usr/share/kubuntu-default-settings"):
+            shutil.copy("/usr/share/kubuntu-default-settings/" +
+                        "dot-gtkrc-2.0-kde4",
+                        os.getenv("HOME") + "/.gtkrc-2.0")
         QDesktopServices.openUrl(QUrl(url))
-        regain_privileges_save()
+        misc.regain_privileges()
+        misc.regain_privileges_save()
 
     def set_language_choices(self, choices, choice_map):
         PageBase.set_language_choices(self, choices, choice_map)
         self.combobox.clear()
-        for choice in choices:
-            self.combobox.addItem(str(choice))
+        self.combobox.addItems(choices)
 
     def set_language(self, language):
         index = self.combobox.findText(str(language))
@@ -553,8 +551,7 @@ class PageKde(PageBase):
             install_medium = i18n.get_string(install_medium, lang)
             for widget in (self.page.try_install_text_label,
                            self.page.try_ubuntu,
-                           self.page.install_ubuntu,
-                           self.page.alpha_warning_label):
+                           self.page.install_ubuntu):
                 text = widget.text()
                 text = text.replace('${RELEASE}', release.name)
                 text = text.replace('${MEDIUM}', install_medium)
@@ -567,21 +564,19 @@ class PageKde(PageBase):
         self.widgetHidden = []
 
     def plugin_set_online_state(self, state):
-        from PyQt4.QtCore import QTimer, SIGNAL
+        from PyQt5.QtCore import QTimer
         if self.page.release_notes_label:
             if state:
                 self.page.release_notes_label.show()
                 QTimer.singleShot(300, self.check_returncode)
                 self.timer = QTimer(self.page)
-                self.timer.connect(
-                    self.timer, SIGNAL("timeout()"), self.check_returncode)
+                self.timer.timeout.connect(self.check_returncode)
                 self.timer.start(300)
             else:
                 self.page.release_notes_label.hide()
 
     def check_returncode(self, *args):
         import subprocess
-        from PyQt4.QtCore import SIGNAL
         if self.wget_retcode is not None or self.wget_proc is None:
             self.wget_proc = subprocess.Popen(
                 ['wget', '-q', _wget_url, '--timeout=15', '--tries=1',
@@ -595,8 +590,7 @@ class PageKde(PageBase):
             else:
                 self.update_installer = False
             self.update_release_notes_label()
-            self.timer.disconnect(
-                self.timer, SIGNAL("timeout()"), self.check_returncode)
+            self.timer.timeout.disconnect(self.check_returncode)
 
     def update_release_notes_label(self):
         lang = self.selected_language()
@@ -660,9 +654,6 @@ class Page(plugin.Plugin):
             except debconf.DebconfError:
                 pass
 
-        show = self.db.get('ubiquity/show_alpha_warning') == 'true'
-        self.ui.set_alpha_warning(show)
-
         localechooser_script = '/usr/lib/ubiquity/localechooser/localechooser'
         if ('UBIQUITY_FRONTEND' in os.environ and
                 os.environ['UBIQUITY_FRONTEND'] == 'debconf_ui'):
