@@ -73,15 +73,40 @@ class PreparePageBase(plugin.PluginUI):
         self.rst_title_text = i18n.get_string('rst_header', lang)
         return
 
+    def db_get(self, question):
+        # This is a hack wrapper to use db.get() not available in PageGtk/PageKde.
+
+        # These are safety nets for the controller attribute (which both have set)
+        # and also _wizard (which is set in its class constructor), just in case.
+        controller = self.get('controller')
+        if controller is None or controller._wizard is None:
+            return None
+
+        # The db should be set when PageGtk/PageKde class constructors run, since
+        # their Wizard's class constructors initialize BaseFrontend, and it calls
+        # start_debconf(), before Wizard instantiates PageGtk/PageKde to get here.
+        # But again, just in case.
+        wizard = controller._wizard
+        db_old = wizard.db
+        if db_old is None:
+            wizard.start_debconf()
+        answer = wizard.db.get(question)
+        if db_old is None:
+            wizard.stop_debconf()
+        return answer
+
 
 class PageGtk(PreparePageBase):
     restricted_package_name = 'ubuntu-restricted-addons'
 
     def __init__(self, controller, *args, **kwargs):
-        if self.is_automatic:
+        self.controller = controller
+        prepare_automatic = self.db_get('ubiquity/prepare_automatic')
+        if self.is_automatic and prepare_automatic != 'true':
+            self.debug('prepare: not included in automatic mode')
             self.page = None
             return
-        self.controller = controller
+
         from ubiquity.gtkwidgets import Builder
         builder = Builder()
         self.controller.add_builder(builder)
@@ -292,10 +317,13 @@ class PageKde(PreparePageBase):
 
     def __init__(self, controller, *args, **kwargs):
         from ubiquity.qtwidgets import StateBox
-        if self.is_automatic:
+        self.controller = controller
+        prepare_automatic = self.db_get('ubiquity/prepare_automatic')
+        if self.is_automatic and prepare_automatic != 'true':
+            self.debug('prepare: not included in automatic mode')
             self.page = None
             return
-        self.controller = controller
+
         try:
             from PyQt5 import uic
             from PyQt5 import QtGui
@@ -462,7 +490,15 @@ class Page(plugin.Plugin):
             with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
                 syslog.syslog(F'ubuntu-drivers list-oem finished with: "{" ".join(f.read().splitlines())}"')
         except FileNotFoundError:
-            syslog.syslog('ubuntu-drivers list-oem finished with no available packages')
+            syslog.syslog("ubuntu-drivers list-oem finished with no available packages. Maybe we need to apt update? "
+                          "Doing that and trying again.")
+            # We only do this when we really have to since it could be slow: apt update & re-run of ubuntu-drivers
+            self.frontend.save_oem_metapackages_list(wait_finished=True)
+            try:
+                with open('/run/ubuntu-drivers-oem.autoinstall', 'r') as f:
+                    syslog.syslog(F'ubuntu-drivers list-oem finished with: "{" ".join(f.read().splitlines())}"')
+            except FileNotFoundError:
+                syslog.syslog("No, we didn't find any OEM packages again.")
 
         if self.should_show_rst_page():
             if not self.ui.show_rst_page():
